package Torus::Driver::mapi;

use strict;
use warnings;

# load configuration-data from ini-file
BEGIN {
  use loadConfig;
}

# we need OLE to access Outlook's Contacts via MAPI
#use Win32::OLE::Strict;
use Win32::OLE;
use Win32::OLE::NLS qw(:LOCALE :DATE);

use Data::Storage::Handler::File qw( s2f );

# dumping data? of course!
use Data::Dumper;

# AppConfig-object from "main"-scope
#my $config = $main::config;

# debugging?
my $DEBUGLEVEL = $config->get("debug_level");
my $DEBUG = ($config->get("debug_level") > 0);

# some values (constants) needed for accessing outlook-folders
my $olFolderList = 2;
my $olFolderDeletedItems = 3;
my $olFolderOutbox = 4;
my $olFolderSentMail = 5;
my $olFolderInbox = 6;
my $olFolderCalendar = 9;
my $olFolderContacts = 10;
my $olFolderJournal = 11;
my $olFolderNotes = 12;
my $olFolderTasks = 13;
my $olFolderDrafts = 16;


# main vars
my $map;
my $map_ldap;

my $mailer;
my $bool_mailerAlreadyRunning;

sub debugLevel {
  my $level_compare = shift;
  return 1 if ($DEBUGLEVEL > $level_compare);
}




sub mailerCheckRunning {
    my $tmp;
    # error-handling
    #$tmp = $mailer->ActiveExplorer->WindowState;
    #If Err.Number = 0 Then mailerCheckRunning = True
    #On Error GoTo 0
    return 0;
}

sub start {
    
    slog("creating mailer-object");
    $mailer = Win32::OLE->new('Outlook.Application') or die "oops - could not instantiate Outlook.Application";
    
    $mailer->Uninitialize();
    #sleep 2;
    
    $mailer->Initialize();
    
    $bool_mailerAlreadyRunning = mailerCheckRunning();
    
    if ($bool_mailerAlreadyRunning) {
        slog("*not* logging on, using running mailer");
    } else {
        if ($config->get("mapi_showProfileChooser")) {
            slog("logging in (using Profile-Chooser), this may take some seconds!");
            $mailer->Session->Logon ('', '', 1);
        } else {
            slog("logging in (auto-selecting profile " . $config->get("mapi_ProfileName") . "), this may take some seconds!");
            $mailer->Session->Logon($config->get("mapi_ProfileName"), $config->get("mapi_ProfilePass"), 0);
        }
    }

}

sub stop {
    
    slog("logging off");
    $mailer->Session->Logoff;
    
    if ($bool_mailerAlreadyRunning || $config->get("mapi_neverCloseOutlook")) {
        slog("*not* quitting running mailer!");
    } else {
        slog("closing active mail-explorer");
        $mailer->ActiveExplorer->Close;
        slog("quitting mailer");
        $mailer->Quit;
    }

    $mailer->Uninitialize();

    slog("destroying mailer-object");
    undef $mailer;

}


# TODO: add functionality to address arbitrary deep folders anywhere in outlook-scope
# TODO: move 'test' to configuration-scope
sub getContactFolder {
  my $foldername = shift;
  slog("getting contacts");
  my $folder;
  if ($foldername) {
    
    # check if it's a expression describing a hierarchical reference
    if (my @address = split(/\./, $foldername)) {
      print "trying to find folder: $foldername", "\n";
      return findFolderByAddress(\@address);
    }
    
    my $rootFolder = $mailer->Session->Folders->GetFirst();
    my $folderlist = $rootFolder->Folders;
    #print Dumper($folderlist);
    for (my $i = 1; $i <= $folderlist->Count; $i++) {
      my $tmpFolder = $folderlist->Item($i);
      #print "folder: ", $tmpFolder->Name, "\n";
      my $name = $tmpFolder->Name;
      if ($name && $name eq $foldername) {
        $folder = $tmpFolder;
        last;
      }
    }
    #exit;
  } else {
    $folder = $mailer->Session->GetDefaultFolder($olFolderContacts);
  }
  return $folder;
}

sub findFolderByAddress {
  my $rootFolder = $mailer->Session->Folders->GetFirst();
  my $folderAddress = shift;
  while (my $levelAddress = shift @$folderAddress) {
    print "diving into: $levelAddress", "\n";
    $rootFolder = findFolder($rootFolder, $levelAddress);
  }
  return $rootFolder;
}

sub findFolder {
  my $parentFolder = shift;
  my $findFolderName = shift;
  my $folderlist = $parentFolder->Folders;
  #print Dumper($folderlist);
  for (my $i = 1; $i <= $folderlist->Count; $i++) {
    my $tmpFolder = $folderlist->Item($i);
    #print "folder: ", $tmpFolder->Name, "\n";
    my $name = $tmpFolder->Name;
    if ($name && $name eq $findFolderName) {
      return $tmpFolder;
    }
  }
}

sub readMapiFolder {
    slog("reading mapi folder");
    my $myFolder = shift;
    my $sourceFolderName = shift;
    my $callback = shift;
    my $options = shift;
    my $maxCount = $myFolder->Items->Count;
    print "Entries: $maxCount", "\n" if (debugLevel(1));
    for (my $i = 1; $i <= $maxCount; $i++) {
      #print "Index: $i", "\n";
      my $contactItem = $myFolder->Items->Item($i);
      #print $contactItem->{NickName}, "\n";
      my $item_raw = readContactItem($contactItem);
      undef $contactItem;
      if ($options->{mapidump}) {
        s2f('../log/mapi_attributes', join("\n", sort keys %$item_raw));
      }
      &$callback($sourceFolderName, $item_raw);
    }
    undef $myFolder;
}

sub readContactItem {
  
  # we get a real OLE MAPI object here, actually an entry from a MAPI ContactFolder
  my $mapiContactObject = shift;

#print Dumper($mapiContactObject);
  
  # build a list of all MAPI field names
  my @mapiFieldNames = sort keys %{$mapiContactObject};
  
  # OLE -> hash - conversion
  # hash to hold all attributes of the MAPI object
  my $contactItem;

    foreach my $mapi_fieldname (@mapiFieldNames) {
      
      #print "mapi-field: $mapi_fieldname", "\n";
      
      # retrieve value of attribute
      my $value = $mapiContactObject->{$mapi_fieldname};
      #my $value;
      #eval('$value = $mapiContactObject->{$mapi_fieldname};');
      
      # skip empty fields
      next if !$value;

      # patches to values
        # bring all date- and time-fields to unified format
        if ($mapi_fieldname eq 'Anniversary' || $mapi_fieldname eq 'Birthday' || $mapi_fieldname eq 'CreationTime') {
          $value = $value->Date('yyyy-MM-dd') . ' ' . $value->Time('HH:mm');
        }
        if ($mapi_fieldname eq 'UserProperties') {
          # ... do more stuff like this here ...
          #my %hash = %{$value};
          #print "====: ", keys %hash, "\n";
          #print Dumper(%hash);
        }
      
      # TODO: convert Win32::OLE - Variants to perlable variables
      
      # filter any (still existing) real Win32::OLE - variables from object, they would make problems in later processing
      next if ($value =~ m/^Win32::OLE/);
  
      # add attribute from object to hash
       $contactItem->{$mapi_fieldname} = $value;
      
    }

  if ( debugLevel(3) ) {
    print Dumper($contactItem);
  }
  
  return $contactItem;
  
}

sub slog {
    my $string = shift;
    print $string, "\n" if ( debugLevel(1) );
}


1;
